#include <jni.h>
#include <string>
#include "ivr4pengine.h"
#include <EGL/egl.h>
#include <EGL/eglext.h>
#include <GLES2/gl2.h>
#include <android/native_window_jni.h>
#include <vector>
#include <unistd.h>
#include <sys/resource.h>
#include "Context.h"

using namespace std;

ANativeWindow * g_pVR4PNativeWindow = NULL;
EGLDisplay      g_mVR4PEglDisplay = NULL;
EGLSurface      g_mVR4PEglSurface = EGL_NO_SURFACE;
EGLContext      g_mVR4PEglContext = EGL_NO_CONTEXT;


volatile int64_t   g_i64ThisFrameIdx = 1;
volatile int64_t   g_i64ResumeFrameIdx = 0;
volatile int64_t   g_i64PauseFrameIdx = 0;
volatile int64_t   g_i64ToRunFrameIdx = 0;
volatile int64_t   g_i64ToStopFrameIdx = 0;

pthread_t           g_android_Render = (pthread_t)NULL;
volatile bool	    g_bVR4PGameEndDV = false;
volatile bool	    g_bVRSensorThread = true;

int32_t           g_iSleepTime = 10;

double GetTimeInSeconds()
{
    struct timespec now;
    clock_gettime( CLOCK_MONOTONIC, &now );
    return ( now.tv_sec * 1e9 + now.tv_nsec ) * 0.000000001;
}

double              g_dTimeForLooping = GetTimeInSeconds();

void  HandleVrModeChanges()
{
    if(g_pVR4PNativeWindow != NULL && g_i64ThisFrameIdx == g_i64ResumeFrameIdx)
    {
        if(g_mVR4PEglDisplay == NULL)
        {
            if(V4_IsFrontBuffer())
            {
                if(!InitContext(true,true))
                {
                    UninitContext(true);
                    if(!InitContext(true,false))
                    {
                        UninitContext(true);
                        V4_UseFrontBuffer(false);
                        InitContext(false,false);
                    }
                }
            }
            else
                InitContext(false,false);
        }
    }
    else if(g_i64ThisFrameIdx == g_i64PauseFrameIdx)
    {
        if(g_mVR4PEglDisplay != NULL)
            UninitContext(V4_IsFrontBuffer());
    }
}

void  MyVRSleepTime(uint32_t usTime)
{
    struct timespec request;
    if (clock_gettime(CLOCK_REALTIME, &request) == -1)
    {
        usleep((useconds_t) usTime);
        return;
    }
    uint64_t next_vsync = request.tv_nsec + ((uint64_t)usTime) * 1000LL;
    request.tv_sec  += next_vsync / 1000000000;
    request.tv_nsec  = (long) (next_vsync % 1000000000);
    clock_nanosleep(CLOCK_REALTIME, TIMER_ABSTIME, &request, NULL);
}

void * android_Render(void *)
{
    g_dTimeForLooping = GetTimeInSeconds();
    double  fCompareTime = double(g_iSleepTime) / 2000000.0;

    g_bVRSensorThread = false;
    setpriority(PRIO_PROCESS, (id_t) gettid(), -18);             //推荐设置在 -10 至 -18 之间，保证主渲染线程的流畅；

    while (!g_bVR4PGameEndDV)
    {
        double fFramwWaitTime = double(V4_CfgGetFrameTimeNaS()) / 1000000000.0;
        if(!V4_IsFrontBuffer() || !V4_CfgGetGlassMode())
        {
            HandleVrModeChanges();
            double thisTime = GetTimeInSeconds();

            //很重要，不要遗漏，多种优化通过改接口实现
            V4_TestThreadBusy();

            if(thisTime - g_dTimeForLooping < fFramwWaitTime - fCompareTime)
            {
                if(g_mVR4PEglDisplay != NULL)
                {
                    MyVRSleepTime((uint32_t) g_iSleepTime);
                }
                else
                {
                    MyVRSleepTime((uint32_t) g_iSleepTime * 1000);
                }
            }
            else
            {
                g_i64ThisFrameIdx++;
                while(thisTime - g_dTimeForLooping >= fFramwWaitTime - fCompareTime * 2.0)
                    g_dTimeForLooping += fFramwWaitTime;

                if(g_mVR4PEglDisplay != NULL)
                {
                    if((g_i64PauseFrameIdx <= g_i64ThisFrameIdx + 2 && g_i64PauseFrameIdx >= g_i64ThisFrameIdx) ||
                       (g_i64ToRunFrameIdx <= g_i64ThisFrameIdx + 2 && g_i64ToRunFrameIdx >= g_i64ThisFrameIdx) ||
                       (g_i64ToStopFrameIdx <= g_i64ThisFrameIdx + 2 && g_i64ToStopFrameIdx >= g_i64ThisFrameIdx) )
                    {
                        glBindFramebuffer(GL_FRAMEBUFFER, 0);
                        glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
                        glClearStencil(0);
                        glClear(GL_COLOR_BUFFER_BIT|GL_STENCIL_BUFFER_BIT);
                        eglSwapBuffers(g_mVR4PEglDisplay,g_mVR4PEglSurface);
                    }
                    else
                    {
                        glBindFramebuffer(GL_FRAMEBUFFER, 0);
                        V4_PreRender();

                        if(!V4_CfgGetGlassMode())
                        {
                            V4_RenderScene(g_iRenderScene_NoGlass);
                        }
                        else
                        {
                            V4_RenderScene(g_iRenderScene_VRLeft);
                            V4_RenderScene(g_iRenderScene_VRRight);
                        }
                        V4_AftRender();
                        eglSwapBuffers(g_mVR4PEglDisplay,g_mVR4PEglSurface);
                    }
                }
                else
                {
                    MyVRSleepTime((uint32_t) g_iSleepTime * 1000);
                }
            }
        }
        else
        {
            HandleVrModeChanges();

            if(!V4_NeedRenderVR())
            {
                if(g_mVR4PEglDisplay != NULL)
                {
                    MyVRSleepTime((uint32_t) g_iSleepTime);
                }
                else
                {
                    MyVRSleepTime((uint32_t) g_iSleepTime * 1000);
                }
            }
            else
            {
                g_i64ThisFrameIdx++;
                if(g_mVR4PEglDisplay != NULL)
                {
                    if((g_i64PauseFrameIdx <= g_i64ThisFrameIdx + 2 && g_i64PauseFrameIdx >= g_i64ThisFrameIdx) ||
                       (g_i64ToRunFrameIdx <= g_i64ThisFrameIdx + 2 && g_i64ToRunFrameIdx >= g_i64ThisFrameIdx) ||
                       (g_i64ToStopFrameIdx <= g_i64ThisFrameIdx + 2 && g_i64ToStopFrameIdx >= g_i64ThisFrameIdx) )
                    {
                        glBindFramebuffer(GL_FRAMEBUFFER, 0);
                        glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
                        glClearStencil(0);
                        glClear(GL_COLOR_BUFFER_BIT|GL_STENCIL_BUFFER_BIT);
                        eglSwapBuffers(g_mVR4PEglDisplay,g_mVR4PEglSurface);
                    }
                    else
                    {
                        glBindFramebuffer(GL_FRAMEBUFFER, 0);

                        V4_PreRender();
                        V4_RenderScene(g_iRenderScene_VRLeft);
                        V4_RenderScene(g_iRenderScene_VRRight);

                        V4_AftRender();
                        eglSwapBuffers(g_mVR4PEglDisplay,g_mVR4PEglSurface);
                    }
                }
                else
                {
                    MyVRSleepTime((uint32_t) g_iSleepTime * 1000);
                }
            }
        }
    }

    g_i64PauseFrameIdx = g_i64ThisFrameIdx;
    HandleVrModeChanges();

    g_bVRSensorThread = true;
    pthread_exit((void *)0);
}

extern "C"
JNIEXPORT void JNICALL
Java_cn_vr4p_vr4pnativesample_JNIWrapper_onStart(JNIEnv *env, jclass type)
{
}

extern "C"
JNIEXPORT void JNICALL
Java_cn_vr4p_vr4pnativesample_JNIWrapper_onResume(JNIEnv *env, jclass type)
{
    if(g_mVR4PEglDisplay == NULL)
        g_i64ResumeFrameIdx = max(g_i64ThisFrameIdx + 1, g_i64PauseFrameIdx + 1);
    g_i64PauseFrameIdx = 0;
    g_i64ToRunFrameIdx = 0;
}

extern "C"
JNIEXPORT void JNICALL
Java_cn_vr4p_vr4pnativesample_JNIWrapper_onPause(JNIEnv *env, jclass type)
{
    g_i64PauseFrameIdx = max(g_i64ThisFrameIdx + 2, g_i64ResumeFrameIdx + 1);
    g_i64ResumeFrameIdx = 0;
    g_i64ToRunFrameIdx = 0;
}

extern "C"
JNIEXPORT void JNICALL
Java_cn_vr4p_vr4pnativesample_JNIWrapper_onStop(JNIEnv *env, jclass type)
{}


extern "C"
JNIEXPORT void JNICALL
Java_cn_vr4p_vr4pnativesample_JNIWrapper_onDestroy(JNIEnv *env, jclass type)
{
    g_bVR4PGameEndDV = true;
    while(!g_bVRSensorThread)
        pthread_join(g_android_Render,NULL);
    g_pVR4PNativeWindow = NULL;
}

extern "C"
JNIEXPORT void JNICALL
Java_cn_vr4p_vr4pnativesample_JNIWrapper_onSvrSurfaceCreated(JNIEnv *env, jclass type, jobject s)
{
    g_pVR4PNativeWindow = ANativeWindow_fromSurface(env,s);
    if(g_mVR4PEglDisplay == NULL)
        g_i64ResumeFrameIdx = max(g_i64ThisFrameIdx + 1, g_i64PauseFrameIdx + 1);
    g_i64PauseFrameIdx = 0;
    g_i64ToRunFrameIdx = 0;
}

extern "C"
JNIEXPORT void JNICALL
Java_cn_vr4p_vr4pnativesample_JNIWrapper_onSvrSurfaceChanged(JNIEnv *env, jclass type, jobject s, jint iWidth, jint iHeight)
{}

extern "C"
JNIEXPORT void JNICALL
Java_cn_vr4p_vr4pnativesample_JNIWrapper_onSvrSurfaceDestroyed(JNIEnv *env, jclass type)
{
    g_pVR4PNativeWindow = NULL;
}

extern "C"
JNIEXPORT void JNICALL
Java_cn_vr4p_vr4pnativesample_JNIWrapper_onCreate(JNIEnv *env, jclass type, jobject obj)
{
    g_i64ThisFrameIdx = 1;
    g_i64ResumeFrameIdx = 0;
    g_i64PauseFrameIdx = 0;
    g_bVR4PGameEndDV = false;

    pthread_create(&g_android_Render, NULL, android_Render, NULL);
}

