package cn.vr4p.vr4pnativesample;

import android.app.Activity;
import android.content.Context;
import android.graphics.PixelFormat;
import android.os.Bundle;
import android.view.Display;
import android.view.SurfaceHolder;
import android.view.WindowManager;
import android.widget.TextView;

import com.vr4p.admin.lib.VREngine;

import java.io.File;

public class VR4PPlayActivity extends Activity implements SurfaceHolder.Callback2
{
    private SurfaceHolder mSurfaceHolder;

    private String  strExtFileDir;

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        VREngine.UseFrontBuffer(true);
        VREngine.InitVsyncPulse(this);

        super.onCreate(savedInstanceState);
        JNIWrapper.onCreate(this);

        getWindow().setFormat(PixelFormat.RGBA_8888);

        getWindow().addFlags( WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON );
        WindowManager.LayoutParams params = getWindow().getAttributes();
        params.screenBrightness = 1.0f;
        getWindow().setAttributes( params );

        ///=====================================================================================
        /// 推荐用takeSurface，可以减少从opengl绘制 到 显示到屏幕上，所延时的时间，这样陀螺仪会更准
        ///
        getWindow().takeSurface(this);

        /*******************************************************************************************
         * 在手机上，如果需要多个 Layout ，完成更多的android UI，可以用下面方法实现
         *
         ViewGroup tempView = (ViewGroup)findViewById(R.id.MyFrameLayout);
         mView2 = new SurfaceView( this );
         tempView.addView(mView2,-1, -1);
         mView2.getHolder().addCallback( this );
         ********************************************************************************************/

        //所有android 手机上，推荐选择这个；
        VREngine.SetVsyncType(VREngine.g_iVsyncType_OneVsyncP);

        Context context = getApplicationContext();
        try
        {
            File fExtFileDir = context.getExternalFilesDir(null);
            assert fExtFileDir != null;
            if(!fExtFileDir.exists())
                fExtFileDir.mkdirs();
            strExtFileDir = fExtFileDir.getAbsolutePath();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }


        Display disp = getWindowManager().getDefaultDisplay();
        VREngine.CfgSetRenderFloatFPSForOP(disp.getRefreshRate());

        //设置VR头盔模式，VR一体机、VR盒子，都必须设置为true；无VR盒子时用手机观看设置为false
        VREngine.CfgSetGlassMode(true); //VREngine.CfgSetGlassMode(false);

        //设置FOV， 这里是弧度，所以需要转化
        VREngine.CfgSetFovV(3.14159265f * 93.5f / 180.0f);

        //设置畸变修正参数，   公式为： 1 + K1 * R * R + K2 * R * R * R * R     K1   K2 为参数；由于R的单位不一样，所以K1 K2的跟你们以前的设置的值可能不一样
        VREngine.CfgSetUndistortParam(0.20f, 0.00f);

        //缩放显示区域的大小（圆的直径），透过头盔刚好看到圆黑边为最佳，既不浪费CPU，又达到视野最大；
        VREngine.CfgSetGlassViewPortScale(1.04f);

        //屏幕显示修正；0表示不旋转，1表示逆时针旋转90度，2表示顺时针旋转90度，3表示旋转180度
        VREngine.CfgSetGlassViewPortRotate(0);

        //设置中间不显示区域的宽度，主要是手机上，VR盒子与手机屏幕有缝，不设置一定宽度，左眼可能透过缝看到右屏，一体机一般可以设置小一些甚至0
        VREngine.CfgSetCenterBlackWidth(40);

        // 打开色散修正功能；
        VREngine.CfgSetColorShift(true);

        //设置显示区域中心点的偏移， 在水平方向主要是与瞳距匹配； 一般CfgComCenterOffsetVByPupiDis返回的就是比较合适的瞳距
        //但如果头盔的瞳距 S1 跟人的瞳距 S0 不一致，还是需要通过调整iOffsetInPixel来进行修正

        VREngine.CfgSetCenterOffset(0, 40);




        Bundle bundle = this.getIntent().getExtras();
        String  strVRVFileName = strExtFileDir;

        //strVRVFileName += "/JauntOne_8K3D.vrb";

        strVRVFileName += "/10K3D_烟袋斜街9k.vrb";

        VREngine.PlayVRV(strVRVFileName,"",0,true,"");
    }


    protected void onDestroy()
    {
        if ( mSurfaceHolder != null )
            JNIWrapper.onSvrSurfaceDestroyed();
        JNIWrapper.onDestroy();

        //不能省略，对兼容新版android；
        VREngine.UnitVsyncPulse(this);

        super.onDestroy();
    }

    @Override protected void onStart()
    {
        super.onStart();
        JNIWrapper.onStart( );
    }

    @Override protected void onResume()
    {
        super.onResume();
        getWindow().setFormat(PixelFormat.RGBA_8888);

        getWindow().addFlags( WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON );
        WindowManager.LayoutParams params = getWindow().getAttributes();
        params.screenBrightness = 1.0f;
        getWindow().setAttributes( params );
        JNIWrapper.onResume( );
    }

    @Override protected void onPause()
    {
        JNIWrapper.onPause( );
        super.onPause();
    }

    @Override protected void onStop()
    {
        JNIWrapper.onStop( );
        super.onStop();
    }

    @Override
    public void surfaceRedrawNeeded(SurfaceHolder holder)
    {}

    @Override
    public void surfaceCreated(SurfaceHolder holder)
    {
        JNIWrapper.onSvrSurfaceCreated(holder.getSurface() );
        mSurfaceHolder = holder;
    }

    @Override
    public void surfaceChanged(SurfaceHolder holder, int format, int width, int height)
    {
        JNIWrapper.onSvrSurfaceChanged(holder.getSurface(), width, height);
        mSurfaceHolder = holder;
    }

    @Override
    public void surfaceDestroyed(SurfaceHolder holder)
    {
        JNIWrapper.onSvrSurfaceDestroyed();
        mSurfaceHolder = null;
    }
}





